def include(context, structure, node):
    if not isinstance(structure, dict):
        context.error(node, 'Bad structure for include directive')

    diff = context.diffnames(structure, ['package', 'filename', 'override'])
    if diff:
        context.error(node, 'Unknown key(s) in "include" directive: %r' % diff)

    package = context.getvalue(structure, 'package')
    if package is not None:
        package = context.resolve(package)
    else:
        package = context.current_package()
    filename = context.getvalue(structure, 'filename', 'configure.yml')
    override = context.getvalue(structure, 'override',
                                context.current_override())
    context.load(filename, package, override)



import unittest

class TestInclude(unittest.TestCase):
    def _callFUT(self, context, structure, node):
        from repoze.configuration.directives import include
        return include(context, structure, node)

    def test_bad_structure(self):
        context = DummyContext()
        structure = 'abc'
        node = DummyNode()
        self.assertRaises(ValueError, self._callFUT, context, structure, node)

    def test_diff(self):
        context = DummyContext(['123'])
        structure = {}
        node = DummyNode()
        self.assertRaises(ValueError, self._callFUT, context, structure, node)
        
    def test_package_not_none_filename_none(self):
        context = DummyContext()
        structure = {'package':'here'}
        node = DummyNode()
        self._callFUT(context, structure, node)
        self.assertEqual(context.loaded, ('configure.yml', 'here', False))

    def test_package_not_none_filename_not_none(self):
        context = DummyContext()
        structure = {'package':'here', 'filename':'here.yml'}
        node = DummyNode()
        self._callFUT(context, structure, node)
        self.assertEqual(context.loaded, ('here.yml', 'here', False))

    def test_package_none_filename_none(self):
        context = DummyContext()
        structure = {}
        node = DummyNode()
        self._callFUT(context, structure, node)
        self.assertEqual(context.loaded, ('configure.yml', 'package', False))

    def test_package_none_filename_not_none(self):
        context = DummyContext()
        structure = {'filename':'here.yml'}
        node = DummyNode()
        self._callFUT(context, structure, node)
        self.assertEqual(context.loaded, ('here.yml', 'package', False))

    def test_withoverride(self):
        context = DummyContext()
        structure = {'filename':'here.yml', 'override':True}
        node = DummyNode()
        self._callFUT(context, structure, node)
        self.assertEqual(context.loaded, ('here.yml', 'package', True))

class DummyContext:
    def __init__(self, diff=None):
        if diff is None:
            diff = []
        self.diff = diff
    
    def getvalue(self, structure, name, default=None):
        return structure.get(name, default)

    def diffnames(self, structure, names):
        return self.diff

    def resolve(self, dottedname):
        return dottedname

    def current_package(self):
        return 'package'

    def current_override(self):
        return False

    def load(self, filename, package, override):
        self.loaded = (filename, package, override)

    def error(self, node, msg):
        raise ValueError(node, msg)
    
class DummyNode:
    pass



import inspect
import sys

from yaml import SafeLoader
from pkg_resources import iter_entry_points

class PluginLoader(SafeLoader):
    EP_GROUP = 'repoze.configuration.directive'
    def __init__(self, context, stream, iter_entry_points=iter_entry_points):
        self.context = context
        for point in list(iter_entry_points(self.EP_GROUP)):
            try:
                directive = point.load()
                self.add_constructor('!' + point.name,
                                     wrap_directive(directive))
            except ImportError:
                pass
        self.add_constructor('tag:yaml.org,2002:str', self.interpolate_str)
        SafeLoader.__init__(self, stream)

    def interpolate_str(self, loader, node):
        value = loader.construct_scalar(node)
        try:
            return self.context.interpolate(value).encode('utf-8')
        except KeyError, why:
            key = why[0]
            exc_info = sys.exc_info()
            msg = ('(Cannot interpolate %%(%s)s while processing lines '
                   '%s:%s-%s:%s of file "%s")' % (
                key,
                node.start_mark.line,
                node.start_mark.column,
                node.end_mark.line,
                node.end_mark.column,
                node.start_mark.name,
                )
            )
            why.args += (msg,)
            raise exc_info[0], why, exc_info[2]
            
def wrap_directive(directive):
    def wrapper(loader, node):
        structure_loader = getattr(loader, 'construct_%s' % node.id)

        if 'deep' in inspect.getargspec(structure_loader)[0]:
            structure = structure_loader(node, deep=True)
        else:
            # scalar structures don't support the 'deep' argument, required
            # for mappings XXX there has to be a better way.
            structure = structure_loader(node)

        context = loader.context

        try:
            infos = directive(context, structure, node)
        except Exception, why:
            exc_info = sys.exc_info()
            msg = ('(while processing lines %s:%s-%s:%s of file "%s")' % (
                node.start_mark.line,
                node.start_mark.column,
                node.end_mark.line,
                node.end_mark.column,
                node.start_mark.name,
                )
            )
            why.args += (msg,)
            raise exc_info[0], why, exc_info[2]

        if isinstance(infos, dict):
            infos = [infos] # b/c

        if infos is not None:
            for info in infos:
                context.action(info, node)

    wrapper.wrapped = directive
    return wrapper





import unittest

class TestLoad(unittest.TestCase):
    def _callFUT(self, filename, package, context):
        from repoze.configuration import load
        return load(filename, package, context)

    def test_no_context(self):
        from repoze.configuration.tests import fixtures
        context = self._callFUT('configure.yml', fixtures, None)
        self.assertEqual(context.registry, {})

    def test_with_context(self):
        context = DummyContext()
        result = self._callFUT('configure.yml', None, context)
        self.assertEqual(result, context)
        self.failUnless(result.loaded, ('configure.yml', None))

class TestExecute(unittest.TestCase):
    def _callFUT(self, filename, package, context):
        from repoze.configuration import execute
        return execute(filename, package, context)

    def test_no_context(self):
        from repoze.configuration.tests import fixtures
        context = self._callFUT('configure.yml', fixtures, None)
        self.assertEqual(context.registry, {})

    def test_with_context(self):
        context = DummyContext()
        result = self._callFUT('configure.yml', None, context)
        self.assertEqual(result, context)
        self.failUnless(result.loaded, ('configure.yml', None))
        self.failUnless(result.executed)

class DummyContext:
    def load(self, filename, package):
        self.loaded = (filename, package)

    def execute(self):
        self.executed = True
        



from repoze.configuration.context import Context

def load(filename='configure.yml', package=None, context=None):
    if context is None:
        registry = {}
        context = Context(registry)
    context.load(filename, package)
    return context

def execute(filename='configure.yml', package=None, context=None):
    context = load(filename, package, context)
    context.execute()
    return context

    
        


    





import pkg_resources

from repoze.configuration.loader import PluginLoader

import os
import re
import sys

_KEYCRE = re.compile(r"%\(([^)]*)\)s")

class ConfigurationError(Exception):
    pass

class Context(object):
    def __init__(self, registry):
        self.registry = registry
        self.actions = []
        self.stack = []
        self.discriminators = {}

    def interpolate(self, value):
        def _interpolation_replace(match):
            s = match.group(1)
            return self.registry[s]
        if '%(' in value:
            value = _KEYCRE.sub(_interpolation_replace, value)
        return value

    def action(self, info, node):
        discriminator = info['discriminator']
        callback = info['callback']
        override = info.get('override')
        stack_override = self.stack and self.stack[-1]['override']
        effective_override = override or stack_override
        if not effective_override:
            if discriminator in self.discriminators:
                conflicting_action = self.discriminators[discriminator]
                raise ConfigurationConflict(node, conflicting_action.node)

        action = Action(discriminator, callback, node)
        self.actions.append(action)
        self.discriminators[discriminator] = action

    def error(self, node, msg):
        msg = '%s\n%s' % (msg , lineinfo(node))
        raise ConfigurationError(msg)

    def resolve(self, dottedname):
        if dottedname.startswith('.') or dottedname.startswith(':'):
            package = self.current_package()
            if not package:
                raise ValueError('name "%s" is irresolveable (no package)' %
                                 dottedname)
            if dottedname in ['.', ':']:
                dottedname = package.__name__
            else:
                dottedname = package.__name__ + dottedname
        try:
            return pkg_resources.EntryPoint.parse(
                'x=%s' % dottedname).load(False)
        except ImportError, why:
            raise ValueError(
                unicode(why) + ' for derived dotted name %s' % dottedname)

    def current_package(self):
        if not self.stack:
            return None
        return self.stack[-1]['package']

    def current_override(self):
        if not self.stack:
            return False
        return self.stack[-1]['override']

    def stream(self, filename, package=None):
        if os.path.isabs(filename):
            return open(filename)
        if package is None:
            package = self.current_package()
        if package is None:
            return open(filename)
        else:
            return pkg_resources.resource_stream(package.__name__, filename)
        
    def execute(self):
        for action in self.actions:
            action.execute()
        return self.registry

    def load(self, filename, package, override=False):
        stream = self.stream(filename, package)
        self.stack.append({'filename':filename, 'package':package,
                           'override':override})
        try:
            loader = PluginLoader(self, stream)
            while loader.check_data():
                loader.get_data()
        finally:
            self.stack.pop()

    def diffnames(self, provided, expected):
        names_provided = set(provided)
        names_expected = set(expected)
        diff = names_provided.difference(names_expected)
        return list(diff)

    def getvalue(self, structure, name, default=None, type=basestring):
        value = structure.get(name, default)
        if value is default:
            return default
        if not isinstance(value, type):
            raise ValueError('"%s" attribute is not a %s: %r' % (
                name, type.__name__, value))
        return value

    def popvalue(self, structure, name, default=None, type=basestring):
        value = structure.pop(name, default)
        if value is default:
            return default
        if not isinstance(value, type):
            raise ValueError('"%s" attribute is not a %s: %r' % (
                name, type.__name__, value))
        return value

    def call_later(self, func, *arg, **kw):
        def callback():
            return func(*arg, **kw)
        return callback

class Action(object):
    def __init__(self, discriminator, callback, node):
        self.discriminator = discriminator
        self.callback = callback
        self.node = node
        
    def execute(self):
        try:
            self.callback()
        except Exception, why:
            exc_info = sys.exc_info()
            msg = 'While executing %s' % lineinfo(self.node)
            why.args += (msg,)
            raise exc_info[0], why, exc_info[2]

class ConfigurationConflict(Exception):
    def __init__(self, node1, node2):
        self.node1 = node1
        self.node2 = node2
        self.msg = str(self)

    def __str__(self):
        message = []
        message.append('Conflicting declarations:')
        message.append(lineinfo(self.node1))
        message.append('conflicts with')
        message.append(lineinfo(self.node2))
        return '\n\n'.join(message)

def lineinfo(node):
    start_mark = node.start_mark
    end_mark = node.end_mark
    filename = start_mark.name
    try:
        f = open(filename, 'r')
        f.seek(start_mark.index)
        data = f.read(end_mark.index - start_mark.index) + 'in '
    except (OSError, IOError):
        data = ''
    msg = ('%slines %s:%s-%s:%s of file "%s"' % (
        data,
        start_mark.line,
        start_mark.column,
        end_mark.line,
        end_mark.column,
        start_mark.name,
        )
    )
    return msg



import unittest

class TestContext(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration import Context
        return Context

    def _makeOne(self, registry=None):
        if registry is None:
            registry = {}
        return self._getTargetClass()(registry)

    def test_ctor(self):
        registry = {}
        context = self._makeOne(registry)
        self.assertEqual(context.registry, registry)
        self.assertEqual(context.stack, [])
        self.assertEqual(context.actions, [])

    def test_interpolate(self):
        registry = {'here':'/here', 'there':'/there'}
        context = self._makeOne(registry)
        result = context.interpolate('Here is %(here)s, there is %(there)s')
        self.assertEqual(result, 'Here is /here, there is /there')

    def test_interpolate_nothing_to_interpolate(self):
        registry = {'here':'/here', 'there':'/there'}
        context = self._makeOne(registry)
        result = context.interpolate('Here is %(here')
        self.assertEqual(result, 'Here is %(here')

    def test_action(self):
        context = self._makeOne()
        context.action({'discriminator':'discriminator',
                        'callback':'callback'}, 'node')
        self.assertEqual(len(context.actions), 1)
        action = context.actions[0]
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.node, 'node')
        self.assertEqual(context.discriminators['discriminator'],
                         context.actions[0])

    def test_action_withconflict(self):
        from repoze.configuration.context import ConfigurationConflict
        context = self._makeOne()
        context.discriminators['discriminator'] = DummyAction()
        context.stack = [{'override':False}]
        self.assertRaises(ConfigurationConflict,
                          context.action,
                          {'discriminator':'discriminator',
                           'callback':'callback'},
                          DummyNode())

    def test_action_withconflict_local_override_no_stack_override(self):
        context = self._makeOne()
        context.discriminators['discriminator'] = DummyAction()
        context.stack = [{'override':False}]
        context.action({'discriminator':'discriminator',
                        'callback':'callback', 'override':True},
                       'node')
        self.assertEqual(len(context.actions), 1)
        action = context.actions[0]
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.node, 'node')
        self.assertEqual(context.discriminators['discriminator'],
                         context.actions[0])

    def test_action_withconflict_stack_override_no_local_override(self):
        context = self._makeOne()
        context.discriminators['discriminator'] = DummyAction()
        context.stack = [{'override':True}]
        context.action({'discriminator':'discriminator',
                        'callback':'callback'},
                       'node')
        self.assertEqual(len(context.actions), 1)
        action = context.actions[0]
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.node, 'node')
        self.assertEqual(context.discriminators['discriminator'],
                         context.actions[0])

    def test_error(self):
        from repoze.configuration.context import ConfigurationError
        context = self._makeOne()
        node = DummyNode()
        self.assertRaises(ConfigurationError, context.error, node, 'message')

    def test_resolve_absolute(self):
        from repoze.configuration.tests.fixtures import fixturefunc
        context = self._makeOne()
        result = context.resolve(
            'repoze.configuration.tests.fixtures:fixturefunc')
        self.assertEqual(result, fixturefunc)
        
    def test_irrresolveable_absolute(self):
        context = self._makeOne()
        self.assertRaises(ValueError, context.resolve,
            'repoze.configuration.tests.fixtures:nonexisting')

    def test_resolve_relative_startswith_colon(self):
        from repoze.configuration.tests import fixtures
        from repoze.configuration.tests.fixtures import fixturefunc
        context = self._makeOne()
        context.stack.append({'package':fixtures})
        result = context.resolve(':fixturefunc')
        self.assertEqual(result, fixturefunc)

    def test_resolve_relative_startswith_dot(self):
        from repoze.configuration import tests
        from repoze.configuration.tests.fixtures import fixturefunc
        context = self._makeOne()
        context.stack.append({'package':tests})
        result = context.resolve('.fixtures:fixturefunc')
        self.assertEqual(result, fixturefunc)

    def test_resolve_relative_is_dot(self):
        from repoze.configuration import tests
        context = self._makeOne()
        context.stack.append({'package':tests})
        result = context.resolve('.')
        self.assertEqual(result, tests)
        
    def test_resolve_relative_nocurrentpackage(self):
        context = self._makeOne()
        self.assertRaises(ValueError, context.resolve, '.fixturefunc')

    def test_irrresolveable_relative(self):
        from repoze.configuration.tests import fixtures
        context = self._makeOne()
        context.stack.append({'package':fixtures})
        self.assertRaises(ValueError, context.resolve, ':notexisting')

    def test_current_package_nostack(self):
        context = self._makeOne()
        self.assertEqual(context.current_package(), None)

    def test_current_package_withstack(self):
        context = self._makeOne()
        context.stack.append({'package':'abc'})
        self.assertEqual(context.current_package(), 'abc')

    def test_current_override_nostack(self):
        context = self._makeOne()
        self.assertEqual(context.current_override(), False)

    def test_current_override_withstack(self):
        context = self._makeOne()
        context.stack.append({'override':True})
        self.assertEqual(context.current_override(), True)
        
    def test_stream_absolute(self):
        import os
        from repoze.configuration.tests import fixtures
        filename = os.path.join(os.path.dirname(
            os.path.abspath(fixtures.__file__)), '__init__.py')
        context = self._makeOne()
        stream = context.stream(filename)
        self.assertEqual(stream.read(), open(filename).read())

    def test_path_nopackage_nocurrentpackage(self):
        import os
        from repoze.configuration.tests import fixtures
        old_cwd = os.getcwd()
        fixtures = os.path.dirname(os.path.abspath(fixtures.__file__))
        filename = os.path.join(fixtures, '__init__.py')
        try:
            os.chdir(fixtures)
            context = self._makeOne()
            stream = context.stream('__init__.py')
            self.assertEqual(stream.read(), open(filename).read())
        finally:
            os.chdir(old_cwd)
        
    def test_path_nopackage_currentpackage(self):
        import os
        from repoze.configuration.tests import fixtures
        context = self._makeOne()
        context.stack.append({'package':fixtures})
        fixtures = os.path.dirname(os.path.abspath(fixtures.__file__))
        filename = os.path.join(fixtures, '__init__.py')
        expected = open(filename).read()
        self.assertEqual(context.stream('__init__.py').read(), expected)

    def test_execute(self):
        registry = {}
        context = self._makeOne(registry)
        actions = [ DummyAction(), DummyAction()]
        context.actions = actions
        result = context.execute()
        self.assertEqual([action.executed for action in actions], [True, True])
        self.failUnless(result is registry)

    def test_load(self):
        context = self._makeOne()
        from repoze.configuration.tests import fixtures
        context.load('configure.yml', fixtures)
        # doesn't blow up

    def test_diffnames(self):
        expected = [1, 2, 3]
        provided = [2]
        context = self._makeOne()
        self.assertEqual(context.diffnames(expected, provided), [1,3])

    def test_getvalue_wrongtype(self):
        context = self._makeOne()
        structure = {'a':1}
        self.assertRaises(ValueError, context.getvalue, structure, 'a')
        
    def test_getvalue_default(self):
        context = self._makeOne()
        structure = {}
        self.assertEqual(context.getvalue(structure, 'a'), None)
        
    def test_getvalue(self):
        context = self._makeOne()
        structure = {'a':'1'}
        self.assertEqual(context.getvalue(structure, 'a'), '1')

    def test_popvalue_wrongtype(self):
        context = self._makeOne()
        structure = {'a':1}
        self.assertRaises(ValueError, context.popvalue, structure, 'a')
        
    def test_popvalue_default(self):
        context = self._makeOne()
        structure = {}
        self.assertEqual(context.popvalue(structure, 'a'), None)
        
    def test_popvalue(self):
        context = self._makeOne()
        structure = {'a':'1'}
        self.assertEqual(context.popvalue(structure, 'a'), '1')
        self.assertEqual(structure, {})

    def test_call_later(self):
        context = self._makeOne()
        class Callback:
            def __call__(self, *arg, **kw):
                self.arg = arg
                self.kw = kw
                return True
        callback = Callback()
        sequence = [1,2,3]
        deferred = context.call_later(callback, 1, 2, name=1, *sequence)
        result = deferred()
        self.assertEqual(result, True)
        self.assertEqual(callback.arg, (1,2,1,2,3))
        self.assertEqual(callback.kw, {'name':1})

class TestAction(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.context import Action
        return Action

    def _makeOne(self, discriminator, callback, node):
        return self._getTargetClass()(discriminator, callback, node)

    def test_ctor(self):
        action = self._makeOne('discriminator', 'callback', 'node')
        self.assertEqual(action.discriminator, 'discriminator')
        self.assertEqual(action.callback, 'callback')
        self.assertEqual(action.node, 'node')

    def test_execute(self):
        class Callback:
            def __call__(self):
                self.called = True
        callback = Callback()
        action = self._makeOne('discriminator', callback, 'node')
        action.execute()
        self.assertEqual(callback.called, True)

    def test_execute_exception(self):
        class Callback:
            def __call__(self):
                raise ValueError('foo')
        callback = Callback()
        node = DummyNode()
        action = self._makeOne('discriminator', callback, node)
        self.assertRaises(ValueError, action.execute)

class TestConfigurationConflict(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.context import ConfigurationConflict
        return ConfigurationConflict

    def _makeOne(self, node1, node2):
        error = self._getTargetClass()(node1, node2)
        return error

    def test_ctor_no_file(self):
        node1 = DummyNode()
        node2 = DummyNode()
        error = self._makeOne(node1, node2)
        self.assertEqual(error.node1, node1)
        self.assertEqual(error.node2, node2)
        lines = error.msg.split('\n')
        self.assertEqual(len(lines), 7)
        self.assertEqual(lines[0], 'Conflicting declarations:')
        self.assertEqual(lines[1], '')
        self.assertEqual(lines[2], 'lines 1:1-1:1 of file "dummy"')
        self.assertEqual(lines[3], '')
        self.assertEqual(lines[4], 'conflicts with')
        self.assertEqual(lines[5], '')
        self.assertEqual(lines[6], 'lines 1:1-1:1 of file "dummy"')

    def test_ctor_with_file(self):
        import os
        here = os.path.normpath(os.path.dirname(__file__))
        fixtures = os.path.join(here, 'fixtures')

        node1 = DummyNode()
        node2 = DummyNode()

        node1.start_mark.line = 1
        node1.end_mark.line = 2
        node1.start_mark.index = 0
        node1.end_mark.index = 50
        file1 = os.path.join(fixtures, 'conflict1.yml')
        node1.start_mark.name = file1

        node2.end_mark.line = 1
        node2.end_mark.line = 2
        node2.start_mark.index = 0
        node2.end_mark.index = 50
        file2 = os.path.join(fixtures, 'conflict2.yml')
        node2.start_mark.name = file2

        error = self._makeOne(node1, node2)

        self.assertEqual(error.node1, node1)
        self.assertEqual(error.node2, node2)

        lines = error.msg.split('\n')
        self.assertEqual(len(lines), 11)
        self.assertEqual(lines[0], 'Conflicting declarations:')
        self.assertEqual(lines[1], '')
        self.assertEqual(lines[2], '--- !abc')
        self.assertEqual(lines[3], 'foo: 1')
        self.assertEqual(lines[4], 'in lines 1:1-2:1 of file "%s"' % file1)
        self.assertEqual(lines[5], '')
        self.assertEqual(lines[6], 'conflicts with')
        self.assertEqual(lines[7], '')
        self.assertEqual(lines[8], '--- !abc')
        self.assertEqual(lines[9], 'foo: 1')
        self.assertEqual(lines[10], 'in lines 1:1-2:1 of file "%s"' % file2)


class DummyMark:
    line = 1
    column = 1
    name = 'dummy'

class DummyNode:
    def __init__(self):
        self.start_mark = DummyMark()
        self.end_mark = DummyMark()

class DummyAction:
    executed = False
    def __init__(self):
        self.node = DummyNode()

    def execute(self):
        self.executed = True
        

        



import unittest

class TestPluginLoader(unittest.TestCase):
    def _getTargetClass(self):
        from repoze.configuration.loader import PluginLoader
        return PluginLoader

    def _makeOne(self, context, stream, iter):
        return self._getTargetClass()(context, stream, iter)

    def test_ctor_load_import_error(self):
        import os
        from repoze.configuration.tests import fixtures
        directory = os.path.dirname(os.path.abspath(fixtures.__file__))
        file = os.path.join(directory, 'configure.yml')
        def directive(context, structure):
            return 'success'
        point = DummyPoint(directive, raise_load_exc=True)
        def iter_entry_points(group, suffix=None):
            yield point
        context = DummyContext()
        loader = self._makeOne(context, open(file), iter_entry_points)
        self.assertEqual(loader.context, context)
        self.failIf('!point' in loader.yaml_constructors) # doesnt blow up

    def test_ctor_ok(self):
        import os
        from repoze.configuration.tests import fixtures
        directory = os.path.dirname(os.path.abspath(fixtures.__file__))
        file = os.path.join(directory, 'configure.yml')
        def directive(context, structure):
            return 'success'
        point = DummyPoint(directive)
        def iter_entry_points(group, suffix=None):
            yield point
        context = DummyContext()
        loader = self._makeOne(context, open(file), iter_entry_points)
        self.assertEqual(loader.context, context)
        self.assertEqual(loader.yaml_constructors['!point'].wrapped, directive)

    def test_interpolate_str(self):
        from yaml.nodes import ScalarNode
        import StringIO
        context = DummyContext()
        loader = DummyLoader(context)
        node = ScalarNode('foo', 'scalar')
        f = StringIO.StringIO()
        loader = self._makeOne(context, f, lambda *arg: ())
        self.assertEqual(loader.interpolate_str(loader, node), 'scalar')
        
    def test_interpolate_str_exc(self):
        from yaml.nodes import ScalarNode
        import StringIO
        context = DummyContext(interpolation_exc=True)
        loader = DummyLoader(context)
        node = ScalarNode('foo', 'scalar', DummyMark(), DummyMark())
        f = StringIO.StringIO()
        loader = self._makeOne(context, f, lambda *arg: ())
        self.assertRaises(KeyError, loader.interpolate_str, loader, node)

class Test_wrap_directive(unittest.TestCase):
    def _callFUT(self, point):
        from repoze.configuration.loader import wrap_directive
        return wrap_directive(point)

    def test_directive_returns_list(self):
        directive = DummyDirective([('a', 'b')])
        constructor = self._callFUT(directive)
        node = DummyNode()
        context = DummyContext()
        loader = DummyLoader(context)
        result = constructor(loader, node)
        self.assertEqual(context.actions, [(('a', 'b'), node)] )

    def test_directive_returns_dict(self):
        directive = DummyDirective({'a':'1'})
        constructor = self._callFUT(directive)
        node = DummyNode()
        context = DummyContext()
        loader = DummyLoader(context)
        result = constructor(loader, node)
        self.assertEqual(context.actions, [({'a':'1'}, node)] )

    def test_withexception(self):
        directive = DummyDirective([('a', 'b')], raise_exc=True)
        constructor = self._callFUT(directive)
        node = DummyNode()
        context = DummyContext()
        loader = DummyLoader(context)
        self.assertRaises(KeyError, constructor, loader, node)

    def test_construct_mapping_uses_deep(self):
        directive = DummyDirective({'a':1})
        constructor = self._callFUT(directive)
        node = DummyNode('mapping')
        context = DummyContext()
        loader = DummyLoader(context)
        constructor(loader, node)
        self.assertEqual(context.actions, [({'a':1}, node)] )
        self.assertEqual(loader.deep, True)

class DummyContext:
    def __init__(self, interpolation_exc=False):
        self.actions = []
        self.interpolation_exc = interpolation_exc
    def action(self, info, node):
        self.actions.append((info, node))
    def interpolate(self, value):
        if self.interpolation_exc:
            raise KeyError(value)
        return value
        
class DummyPoint:
    name = 'point'
    def __init__(self, directive, raise_load_exc=False):
        self.directive = directive
        self.raise_load_exc = raise_load_exc

    def load(self):
        if self.raise_load_exc:
            raise ImportError('foo')
        return self.directive

class DummyDirective:
    def __init__(self, result, raise_exc=False):
        self.result = result
        self.raise_exc = raise_exc
        
    def __call__(self, context, structure, node):
        if self.raise_exc:
            raise KeyError('yo')
        return self.result

class DummyLoader:
    def __init__(self, context):
        self.context = context

    def construct_theid(self, node):
        return {}

    def construct_mapping(self, node, deep=False):
        self.deep = deep
        return {}

    def construct_scalar(self, node):
        return 'scalar'

class DummyMark:
    line = 1
    column = 1
    name = 'dummy'
    
class DummyNode:
    def __init__(self, id='theid'):
        self.id = id
        self.start_mark = DummyMark()
        self.end_mark = DummyMark()
        
        
